/***************************************************************************
 *   Copyright (C) 2004 by Francisco J. Ros                                *
 *   fjrm@dif.um.es                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

///
/// \file	OLSR_pkt.h
/// \brief	This file contains all declarations of %OLSR packets and messages,
///		including all related contants and macros.
///

#ifndef __OLSR_pkt_h__
#define __OLSR_pkt_h__

#include <packet.h>

/********** Message types **********/

/// %OLSR HELLO message type.
#define OLSR_HELLO_MSG		1
/// %OLSR TC message type.
#define OLSR_TC_MSG		2
/// %OLSR MID message type.
#define OLSR_MID_MSG		3

/******** New Message types for encryption, Ashraf *********/
#define SOLSR
#define OLSR_ENC_HELLO_MSG		130
#define OLSR_ENC_TC_MSG			131
#define OLSR_ENC_CHALLENGE_MSG	132
#define OLSR_ENC_RESPONSE_MSG	133
#define OLSR_ENC_KEY_MSG		134

/********** Packets stuff **********/

/// Accessor to the %OLSR packet.
#define PKT_OLSR(p)		OLSR_pkt::access(p)

///
/// \brief Size of the addresses we are using.
///
/// You should always use this macro when interested in calculate addresses'
/// sizes. By default IPv4 addresses are supposed, but you can change to
/// IPv6 ones by recompiling with OLSR_IPv6 constant defined.
///
#ifdef OLSR_IPv6
#define ADDR_SIZE	16
#else
#define ADDR_SIZE	4
#endif

/// Maximum number of messages per packet.
#define OLSR_MAX_MSGS		4

/// Maximum number of hellos per message (4 possible link types * 3 possible nb types).
#define OLSR_MAX_HELLOS		12

/// Maximum number of addresses advertised on a message.
#define OLSR_MAX_ADDRS		64

/// Size (in bytes) of packet header.
#define OLSR_PKT_HDR_SIZE	4

/// Size (in bytes) of message header.
#define OLSR_MSG_HDR_SIZE	12

/// Size (in bytes) of hello header.
#define OLSR_HELLO_HDR_SIZE	4

/// Size (in bytes) of hello_msg header.
#define OLSR_HELLO_MSG_HDR_SIZE	4

/// Size (in bytes) of tc header.
#define OLSR_TC_HDR_SIZE	4

// <ashraf>
// all size are in Bytes.
///maximum number of key blocks in a ENC_HELLO or TC messages
#define OLSR_ENC_HELLO_HDR_SIZE		4
#define OLSR_KEY_ID_SIZE	4
#define OLSR_MAX_KB		16
//size in bytes, 32x8=256
#define OLSR_MAX_MAC_LENGTH		32
#define OLSR_MAX_SESSION_KEY_LENGTH	16
//assuming 32 bits/4 bytes  address.the maximum size of a HELLO message. size in bytes
#define OLSR_MAX_ENC_HELLO_BODY 	OLSR_HELLO_HDR_SIZE + (OLSR_HELLO_MSG_HDR_SIZE + OLSR_MAX_ADDRS*4)*OLSR_MAX_HELLOS
//assumption on certificate length, size in bytes, 256*8= 2048 bits
#define OLSR_MAX_PUBLIC_KEY_LENGTH	256
#define OLSR_MAX_CERT_LENGTH		2*OLSR_MAX_PUBLIC_KEY_LENGTH
#define OLSR_MAX_SIGN_LENGTH		OLSR_MAX_PUBLIC_KEY_LENGTH
//</ashraf>

/// Auxiliary struct which is part of the %OLSR HELLO message (struct OLSR_hello).
typedef struct OLSR_hello_msg {

        /// Link code.
	u_int8_t	link_code_;
	/// Reserved.
	u_int8_t	reserved_;
	/// Size of this link message.
	u_int16_t	link_msg_size_;
	/// Number of interface addresses contained in nb_iface_addrs_.
	int		count;	
	/// List of interface addresses of neighbor nodes.
	nsaddr_t	nb_iface_addrs_[OLSR_MAX_ADDRS];
	
	inline u_int8_t&	link_code()		{ return link_code_; }
	inline u_int8_t&	reserved()		{ return reserved_; }
	inline u_int16_t&	link_msg_size()		{ return link_msg_size_; }
	inline nsaddr_t&	nb_iface_addr(int i)	{ return nb_iface_addrs_[i]; }
	
	inline u_int32_t size() { return OLSR_HELLO_MSG_HDR_SIZE + count*ADDR_SIZE; }

} OLSR_hello_msg;

/// %OLSR HELLO message.
typedef struct OLSR_hello {

	/// Reserved.
	u_int16_t	reserved_;
	/// HELLO emission interval in mantissa/exponent format.
	u_int8_t	htime_;
	/// Willingness of a node for forwarding packets on behalf of other nodes.
	u_int8_t	willingness_;	
	/// Number of OLSR_hello_msg contained in hello_body_.
	int		count;
	/// List of OLSR_hello_msg.
	OLSR_hello_msg	hello_body_[OLSR_MAX_HELLOS];
	
	
	
	inline u_int16_t&	reserved()		{ return reserved_; }
	inline u_int8_t&	htime()			{ return htime_; }
	inline u_int8_t&	willingness()		{ return willingness_; }
	inline OLSR_hello_msg&	hello_msg(int i)	{ return hello_body_[i]; }
	
	inline u_int32_t size() {
		u_int32_t sz = OLSR_HELLO_HDR_SIZE;
		for (int i = 0; i < count; i++)
			sz += hello_msg(i).size();
		return sz;
	}
	
} OLSR_hello;

// the following data structures are defined new. <ashraf>
/*
 *	|-------------------------------------------|
 *	|			key_id							|
 *	|-------------------------------------------|
 *	|			encrypted session_key			|
 *	|-------------------------------------------|
 *
*/
typedef struct OLSR_key_block{
	u_int32_t 	key_id_;	
	typedef char 	session_key_t[OLSR_MAX_SESSION_KEY_LENGTH];
	session_key_t 	enc_session_key_;
	
	inline u_int32_t& 	key_id()	{ return key_id_;}
	inline session_key_t& 	enc_session_key()	{return enc_session_key_;}
	
	inline u_int32_t size() {
		u_int32_t sz	=	OLSR_KEY_ID_SIZE;
		sz				+=	OLSR_MAX_SESSION_KEY_LENGTH;
		return sz;
	}
	
} OLSR_key_block;

/*	|0-------------------16-------------------31|
 *	|	KB_counter		|  MAC_length			|
 *	|-------------------------------------------|
 *	|			keyblock 1						|
 *	|-------------------------------------------|
 *	|			keyblock 2						|
 *	|-------------------------------------------|
  *	|			....							|
 *	|-------------------------------------------|
 *	|			keyblock KB_counter				|
 *	|-------------------------------------------|
 *	|			MAC								|
 *	|-------------------------------------------|
 *	|			Encrypted hello					|
 *	|-------------------------------------------|
 */
typedef struct OLSR_enc_hello{
	///General
	u_int16_t KB_counter_;  // number of key block in this message
	u_int16_t MAC_length_;	
	OLSR_key_block key_block_[OLSR_MAX_KB];	
		typedef char MAC_t [OLSR_MAX_MAC_LENGTH];
	MAC_t MAC_;
	
	// orginally this will contain encrypted body. but right now
	// we will send only plain hello msg packed in enc_hello msg.	
	/*
	typedef char enc_body_t[OLSR_MAX_ENC_HELLO_BODY];
	enc_body_t enc_body_;
	*/
	OLSR_hello hello_;
	
	
	inline u_int16_t&	KB_counter()	{ return KB_counter_; }
	inline u_int16_t&	MAC_length()	{ return MAC_length_; }
	inline MAC_t&		MAC()		{ return MAC_; }
	//inline enc_body_t&	enc_body()	{ return enc_body_; }
	inline	OLSR_hello&	hello()		{ return hello_; }
	inline OLSR_key_block&	key_block(u_int16_t i)		{return key_block_[i];}
		
	inline u_int32_t size() {
		u_int32_t sz = OLSR_ENC_HELLO_HDR_SIZE; //4
		for (int i = 0; i < KB_counter_; i++)
			sz += key_block(i).size();
		sz+= OLSR_MAX_MAC_LENGTH;		// 8B=256b
		sz+= hello_.size();
		return sz;
	}	
} OLSR_enc_hello;

/*	|0-------------------16-------------------31|
 *	|	ANSN			|			reserved	|
 *	|-------------------------------------------|
 *	|		Node description					|
 *	|-------------------------------------------|
 *	|			Adv ng main addr				|
 *	|-------------------------------------------|
  *	|			Adv ng main addr				|
 *	|-------------------------------------------|
 *	|			...								|
 *	|-------------------------------------------|
  */
typedef struct OLSR_enc_tc_msg {
        /// Advertised Neighbor Sequence Number.
	u_int16_t	ansn_;	
	u_int16_t	reserved_;
	// bit string representing the node description extension of the advertising nodes certificate
	u_int32_t	node_description_;
	/// List of neighbors' main addresses.
	nsaddr_t	nb_main_addrs_[OLSR_MAX_ADDRS];
	/// Number of neighbors' main addresses contained in nb_main_addrs_.
	int		count;
        
	inline	u_int16_t&	ansn()			{ return ansn_; }
	inline	u_int16_t&	reserved()		{ return reserved_; }
	inline	u_int32_t&	node_description()		{ return node_description_; }
	inline	nsaddr_t&	nb_main_addr(int i)	{ return nb_main_addrs_[i]; }
	
	inline	u_int32_t size() { return OLSR_TC_HDR_SIZE + 4 + count*ADDR_SIZE; } // node descipt = 4

} OLSR_enc_tc_msg;

// TC and Hello are common
typedef struct OLSR_enc_tc{
	u_int16_t KB_counter_;  // number of key block in this message	
	u_int16_t MAC_length_;	
	OLSR_key_block key_block_[OLSR_MAX_KB];	
		typedef char MAC_t [OLSR_MAX_MAC_LENGTH];
	MAC_t MAC_;
	
	// orginally this will contain encrypted body. but right now
	// we will send only plain tc msg.	
	/*
	typedef char enc_body_t[OLSR_MAX_ENC_TC_BODY];
	enc_body_t enc_body_;
	*/
	OLSR_enc_tc_msg tc_;
	
	inline u_int16_t&	KB_counter()	{ return KB_counter_; }	
	inline u_int16_t&	MAC_length()	{ return MAC_length_; }
	inline MAC_t&		MAC()		{ return MAC_; }
	//inline enc_body_t&	enc_body()	{ return enc_body_; }
	inline	OLSR_enc_tc_msg&	tc()		{ return tc_; }
	inline OLSR_key_block&	key_block(u_int16_t i)		{return key_block_[i];}
		
	inline u_int32_t size() {
		u_int32_t sz = OLSR_ENC_HELLO_HDR_SIZE; // 4B = 32b,
		for (int i = 0; i < KB_counter_; i++)
			sz += key_block(i).size();
		sz+= OLSR_MAX_MAC_LENGTH;
		sz+= tc_.size();
		return sz;
	}
	
} OLSR_enc_tc;

/*	----------------------------------------------
 *	|	reserved		|		cert length		|
 *	---------------------------------------------
 *	|		key_id								|
 *	---------------------------------------------
 *	|			certificate						|
	--------------------------------------------- */
typedef struct OLSR_enc_challenge{
	u_int16_t reserved_;
    u_int16_t cert_length_;
	u_int32_t key_id_;
	//u_int32_t nonce_;
	typedef char incumbent_cert_t[OLSR_MAX_CERT_LENGTH];
	incumbent_cert_t incumbent_cert_;
		
	inline u_int16_t&	reserved()	{return reserved_;}
	inline u_int16_t&   cert_length()	{ return cert_length_;}	
	inline u_int32_t& key_id()	{return key_id_;}
	//inline u_int32_t& nonce()	{return nonce_;}
	inline incumbent_cert_t& incumbent_cert()	{return incumbent_cert_;}
	
	inline u_int32_t size() {
		u_int32_t sz = 4;//header
		sz+=4;	//key id
		sz+=OLSR_MAX_CERT_LENGTH;
		return sz;
	}	
} OLSR_enc_challenge;
	
/*	--------------------------------------------|
 *	|	SigLength		|		CertLength		|
 *	|-------------------------------------------|
 *	|				ckey_id						|
 *	|-------------------------------------------|
 *	|				rkey_id						|
 *	|-------------------------------------------|
 *	|				signature					|
 *	|-------------------------------------------|
 *	|				certificate					|
	|-------------------------------------------| */

typedef struct OLSR_enc_response{
	u_int16_t sign_len_;
	u_int16_t cert_len_;
	u_int32_t ckey_id_;
	u_int32_t rkey_id_;
		
	typedef char joiner_sign_t[OLSR_MAX_SIGN_LENGTH];
	joiner_sign_t joiner_sign_;
	
	typedef char joiner_cert_t[OLSR_MAX_CERT_LENGTH];
	joiner_cert_t joiner_cert_;
	
	inline u_int16_t&	sign_len()	{return sign_len_;}
	inline u_int16_t&	cert_len()	{return cert_len_;}
	inline u_int32_t& 	ckey_id()	{return ckey_id_;}
	inline u_int32_t& 	rkey_id()	{return rkey_id_;} 	
	inline joiner_sign_t& 	joiner_sign()	{return joiner_sign_;}
	inline joiner_cert_t& 	joiner_cert()	{return joiner_cert_;}	
	
	inline u_int32_t size() {
		u_int32_t sz=0;
		sz+=4; // sign and cert length
		sz+= 8;//A and B key id		
		sz+=OLSR_MAX_SIGN_LENGTH;
		sz+=OLSR_MAX_CERT_LENGTH;//joiner certificate
		return sz;
	}
	
} OLSR_enc_response;

/*	--------------------------------------------|
 *	|	key_length		|		sign_length		|
 *	|-------------------------------------------|
 *	|				ckey_id						|
 *	|-------------------------------------------|
 *	|				enc_key						|
 *	|-------------------------------------------|
 *	|				signature					|
	|-------------------------------------------| */
typedef struct OLSR_enc_key{
	u_int16_t	key_length_;
	u_int16_t	sign_length_;
	u_int32_t key_id_;
	//session key encrypted by A's public key	
	typedef char enc_key_t[OLSR_MAX_PUBLIC_KEY_LENGTH];
	enc_key_t enc_key_;	
	
	typedef char signature_t[OLSR_MAX_SIGN_LENGTH]; // B's signature
	signature_t signature_;
	
	inline u_int16_t& key_length()	{return key_length_;}
	inline u_int16_t& sign_length()	{return sign_length_;}
	inline u_int32_t& key_id()	{return key_id_;}
	inline enc_key_t& enc_key()	{return enc_key_;}
	inline signature_t& signature()	{return signature_;}
		
	inline u_int32_t size() {
		u_int32_t sz =	4;	// header
		sz+=	OLSR_KEY_ID_SIZE;
		sz+=	OLSR_MAX_PUBLIC_KEY_LENGTH;		
		sz+=	OLSR_MAX_SIGN_LENGTH;//B's signature		
		return sz;
	}
	
}OLSR_enc_key;
// end of new declarations </ashraf>

/// %OLSR TC message.
typedef struct OLSR_tc {
        /// Advertised Neighbor Sequence Number.
	u_int16_t	ansn_;
	/// Reserved.
	u_int16_t	reserved_;
	/// List of neighbors' main addresses.
	nsaddr_t	nb_main_addrs_[OLSR_MAX_ADDRS];
	/// Number of neighbors' main addresses contained in nb_main_addrs_.
	int		count;
        
	inline	u_int16_t&	ansn()			{ return ansn_; }
	inline	u_int16_t&	reserved()		{ return reserved_; }
	inline	nsaddr_t&	nb_main_addr(int i)	{ return nb_main_addrs_[i]; }
	
	inline	u_int32_t size() { return OLSR_TC_HDR_SIZE + count*ADDR_SIZE; }

} OLSR_tc;

/// %OLSR MID message.
typedef struct OLSR_mid {

	/// List of interface addresses.
	nsaddr_t	iface_addrs_[OLSR_MAX_ADDRS];
	/// Number of interface addresses contained in iface_addrs_.
	int		count;
	
	inline nsaddr_t&	iface_addr(int i)	{ return iface_addrs_[i]; }
	
	inline u_int32_t	size()			{ return count*ADDR_SIZE; }
	
} OLSR_mid;

/// %OLSR message.
 struct OLSR_msg {

	u_int8_t	msg_type_;	///< Message type.
	u_int8_t	vtime_;		///< Validity time.
	u_int16_t	msg_size_;	///< Message size (in bytes).
	nsaddr_t	orig_addr_;	///< Main address of the node which generated this message.
	u_int8_t	ttl_;		///< Time to live (in hops).
	u_int8_t	hop_count_;	///< Number of hops which the message has taken.
	u_int16_t	msg_seq_num_;	///< Message sequence number.
	union {
		OLSR_hello	hello_;
		OLSR_tc		tc_;
		OLSR_mid	mid_;
		//new message type, ashraf
		OLSR_enc_hello	enc_hello_;
		OLSR_enc_tc	enc_tc_;
		OLSR_enc_challenge	enc_challenge_;
		OLSR_enc_response	enc_response_;
		OLSR_enc_key	enc_key_;
	} msg_body_;			///< Message body.
	
	inline	u_int8_t&	msg_type()	{ return msg_type_; }
	inline	u_int8_t&	vtime()		{ return vtime_; }
	inline	u_int16_t&	msg_size()	{ return msg_size_; }
	inline	nsaddr_t&	orig_addr()	{ return orig_addr_; }
	inline	u_int8_t&	ttl()		{ return ttl_; }
	inline	u_int8_t&	hop_count()	{ return hop_count_; }
	inline	u_int16_t&	msg_seq_num()	{ return msg_seq_num_; }
	inline	OLSR_hello&	hello()		{ return msg_body_.hello_; }
	inline	OLSR_tc&	tc()		{ return msg_body_.tc_; }
	inline	OLSR_mid&	mid()		{ return msg_body_.mid_; }
	
	//added, ashraf
	inline	OLSR_enc_hello&		enc_hello()		{ return msg_body_.enc_hello_; }
	inline	OLSR_enc_tc&		enc_tc()		{ return msg_body_.enc_tc_; }
	inline	OLSR_enc_challenge&	enc_challenge()		{ return msg_body_.enc_challenge_; }
	inline	OLSR_enc_response&	enc_response()		{ return msg_body_.enc_response_; }
	inline	OLSR_enc_key&		enc_key()		{ return msg_body_.enc_key_; }
	
	inline u_int32_t size() {
		u_int32_t sz = OLSR_MSG_HDR_SIZE;
		if (msg_type() == OLSR_HELLO_MSG)
			sz += hello().size();
		else if (msg_type() == OLSR_TC_MSG)
			sz += tc().size();
		else if (msg_type() == OLSR_MID_MSG)
			sz += mid().size();
		//added, ashraf
		else if (msg_type() == OLSR_ENC_HELLO_MSG)
			sz += enc_hello().size();
		else if (msg_type() == OLSR_ENC_TC_MSG)
			sz += enc_tc().size();
		else if (msg_type() == OLSR_ENC_CHALLENGE_MSG)
			sz += enc_challenge().size();
		else if (msg_type() == OLSR_ENC_RESPONSE_MSG)
			sz += enc_response().size();
		else if (msg_type() == OLSR_ENC_KEY_MSG)
			sz += enc_key().size();
		
		return sz;
	}

} ;

/// %OLSR packet.
typedef struct OLSR_pkt {

	u_int16_t	pkt_len_;			///< Packet length (in bytes).
	u_int16_t	pkt_seq_num_;			///< Packet sequence number.
	OLSR_msg	pkt_body_[OLSR_MAX_MSGS];	///< Packet body.
	int		count;				///< Number of OLSR_msg contained in pkt_body_.
	
	inline	u_int16_t&	pkt_len()	{ return pkt_len_; }
	inline	u_int16_t&	pkt_seq_num()	{ return pkt_seq_num_; }
	inline	OLSR_msg&	msg(int i)	{ return pkt_body_[i]; }
			
	static int offset_;
	inline static int& offset() { return offset_; }
	inline static struct OLSR_pkt* access(const Packet* p) {
		return (struct OLSR_pkt*)p->access(offset_);
	}

} OLSR_pkt;

#endif
